<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use App\Http\Controllers\Traits\Functions;
use App\Models\Notifications;
use App\Models\Subscriptions;
use App\Models\Plans;
use App\Models\TaxRates;
use App\Models\PaymentGateways;
use Cardinity\Client;
use Cardinity\Method\Payment;
use Cardinity\Exception;

class RebillCardinity implements ShouldQueue
{
  use Dispatchable, InteractsWithQueue, Queueable, SerializesModels, Functions;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
      // Get Payment Gateway
      $payment = PaymentGateways::whereName('Cardinity')->firstOrFail();

      $subscriptions = Subscriptions::where('ends_at', '<', now())
      ->whereNotNull('payment_id')
      ->whereCancelled('no')
      ->get();

      if ($subscriptions) {

        foreach ($subscriptions as $subscription) {

          // Get price of Plan
          $plan = Plans::whereName($subscription->stripe_price)->first();

          // Get Taxes
          $taxes = TaxRates::whereIn('id', collect(explode('_', $subscription->taxes)))->get();
          $totalTaxes = ($plan->price * $taxes->sum('percentage') / 100);
          $planPrice = ($plan->price + $totalTaxes);

          if ($subscription->subscribed()->free_subscription == 'no') {

            $client = Client::create([
              'consumerKey' => $payment->key,
              'consumerSecret' => $payment->key_secret,
            ]);
  
          // Payment ID Subscription
          $paymentId = $subscription->payment_id;
  
          $method = new Payment\Create([
              'amount' => $planPrice,
              'currency' => config('settings.currency_code'),
              'settle' => false,
              'description' => __('general.subscription_for').' @'.$subscription->subscribed()->username,
              'order_id' => str_random(5),
              'country' => $subscription->subscribed()->country()->country_code,
              'payment_method' => Payment\Create::RECURRING,
              'payment_instrument' => [
                  'payment_id' => $paymentId
              ],
          ]);
  
          try {
              /** @type Cardinity\Method\Payment\Payment */
              $payment = $client->call($method);
  
              $status = $payment->getStatus();
  
              $newPaymentId = $payment->getId();
  
              if ($status == 'approved') {

                // Admin and user earnings calculation
                $earnings = $this->earningsAdminUser($subscription->subscribed()->custom_fee, $plan->price, null, null);

                // Insert Transaction
                $this->transaction(
                  $newPaymentId,
                  $subscription->user()->id,
                  $subscription->id,
                  $subscription->subscribed()->id,
                  $plan->price,
                  $earnings['user'],
                  $earnings['admin'],
                  'Cardinity',
                  'subscription',
                  $earnings['percentageApplied'],
                  $subscription->taxes
                );

                // Add Earnings to Creator
                $subscription->subscribed()->increment('balance', $earnings['user']);

                // Send Notification to User --- destination, author, type, target
                Notifications::send($subscription->subscribed()->id, $subscription->user()->id, 12, $subscription->user()->id);

                $subscription->update([
                  'ends_at' => $subscription->subscribed()->planInterval($plan->interval)
                ]);
              }
  
            } catch (Exception\Declined $exception) {
                \Log::info('Cardinity Renewal Failed - ' . $exception->getErrorsAsString());

                $subscription->update([
                  'cancelled' => 'yes'
              ]);
    
            } catch (Exception\ValidationFailed $exception) {
                \Log::info('Cardinity Renewal Failed - ' . $exception->getErrorsAsString());

                $subscription->update([
                  'cancelled' => 'yes'
              ]);
          }
              catch (\Exception $exception) {
                \Log::info('Cardinity Renewal Failed - ' . $exception->getMessage());

                $subscription->update([
                  'cancelled' => 'yes'
              ]);
          }

         }
        }
      }

    }
}
